local HELM, NECK, SHOULDER, SHIRT, CHEST, WAIST, LEGS, FEET, WRISTS, HANDS, RING1, RING2, TRINK1, TRINK2, BACK, WEP, OFFHAND = 1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17
local lastInspectTime = GetTime()

local_, GridStatusItemLevel = ...

if not GridStatusItemLevel.L then GridStatusItemLevel.L = { } end

local L = setmetatable(GridStatusItemLevel.L, {
	__index = function(t, k)
		t[k] = k
		return k
	end
})

local cache = {}

local ItemUpgradeInfo = LibStub("LibItemUpgradeInfo-1.0")
local GridStatusItemLevel = Grid:GetModule("GridStatus"):NewModule("GridStatusItemLevel", "AceTimer-3.0")

GridStatusItemLevel.menuName = L["Item Level"];

GridStatusItemLevel.defaultDB = {
	alert_ilvl = {
		text = L["Item Level"],
		enable = true,
		priority = 99,
		range = false,
		color = {r = 192, g = 192, b = 192, a = 1},
	},
}

hooksecurefunc("NotifyInspect", function() lastInspectTime = GetTime(); end)

function GridStatusItemLevel:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatuses()
end

function GridStatusItemLevel:OnEnable()
	local refreshTimer = self:ScheduleRepeatingTimer("UpdateAllUnits", 0)
	local clearCacheTimer = self:ScheduleRepeatingTimer("ClearCache", 7200) --we purge the cache every 2 hours
end

function GridStatusItemLevel:RegisterStatuses()
	self:RegisterStatus("alert_ilvl", L["Item Level"], alert_ilvl)
end

function GridStatusItemLevel:UnregisterStatuses()
	self:UnregisterStatus("alert_ilvl")
end

function GridStatusItemLevel:ClearCache()
	for k,v in pairs(cache) do cache[k]=nil end
end

local InspectFrame = CreateFrame("Frame", "InspectFrame")
InspectFrame:Hide()


function InspectEvent(self, event, guid)
	if(not cache[guid]) then
		return
	end

	if(guid == UnitGUID(cache[guid].uid)) then --make sure the event is for the player we expect
		CalculateItemLevel(guid, cache[guid].uid)
		InspectFrame:UnregisterEvent("INSPECT_READY")
	end
end
InspectFrame:SetScript("OnEvent", InspectEvent)


local GridRoster = Grid:GetModule("GridRoster")
function GridStatusItemLevel:UpdateAllUnits()
	for guid, unitid in GridRoster:IterateRoster() do
			GridStatusItemLevel:UpdateUnit(guid, unitid)
	end
end

function GridStatusItemLevel:UpdateUnit(guid, unitid)
	local settings = GridStatusItemLevel.db.profile.alert_ilvl
	local itemLevel = 0

	if(not cache[guid] or (cache[guid].itemLevel == 0)) then 
		if(CheckInteractDistance(unitid, 1) and CanInspect(unitid)) then
			if((GetTime() - lastInspectTime) > 2) then
				cache[guid] = {itemLevel = 0, uid = unitid}
				InspectFrame:RegisterEvent("INSPECT_READY")
				lastInspectTime = GetTime()
				NotifyInspect(unitid)
			end
		end
	else
		itemLevel = cache[guid].itemLevel
		SendUpdate(guid, unitid, itemLevel)
	end
end

function SendUpdate(guid, unitid, itemLevel)
	local settings = GridStatusItemLevel.db.profile.alert_ilvl

	if settings.enable then
		GridStatusItemLevel.core:SendStatusGained(guid, "alert_ilvl", 
			settings.priority, 
			(settings.range and 40),
			settings.color,
			string.format("%d", itemLevel)
		)
	else
		if GridStatusItemLevel.core:GetCachedStatus(guid, "alert_ilvl") then GridStatusItemLevel.core:SendStatusLost(guid, "alert_ilvl") end
	end
end

function CalculateItemLevel(guid, unitid)
	local totalIlvl, avgIlvl = 0
	local iter_min, iter_max = 0
	local itemLevel = 0
	local equipType = 0

	local count = 0
	for i = 1,17 do
		if(i ~= SHIRT) then
			local item = GetInventoryItemLink(unitid, i)
			if(item) then
				count = count + 1
				local upgradeID = ItemUpgradeInfo:GetUpgradeID(item)
				local upgrade = ItemUpgradeInfo:GetCurrentUpgrade(upgradeID)
				_,_,_,itemLevel = GetItemInfo(item)

				if(upgrade == 1) then 
					itemLevel = itemLevel + 4
				elseif(upgrade == 2) then
					itemLevel = itemLevel + 8
				end
				totalIlvl = totalIlvl + itemLevel
			end
		end
	end
		if(count < 15) then --often we miss an item, if we did we set item level = 0 so we know to recalculate
			avgIlvl = 0
			cache[guid].itemLevel = avgIlvl
		else
			avgIlvl = math.floor(totalIlvl / count)
			cache[guid].itemLevel = avgIlvl
			SendUpdate(guid, unitid, avgIlvl)
		end
end